module Main (main) where

import Text.Parsec

import Lib (exprparser, evaluate, replaceVars, Expr)

import Data.Map (Map)
import qualified Data.Map as Map
import Data.Ratio

import System.IO

main :: IO ()
main = ioLoop

precision = 5 :: Int

showRatio :: Int -> Rational -> String
showRatio p r = (if (r < 0) then "-" else "") ++ prepoint_digits ++ (if (length postpoint_digits > 0) then ("." ++ postpoint_digits) else "")
    where
    	prepoint_digits  = init . show . round . abs $ (r * 10)
	postpoint_digits = reverse .dropWhile (=='0') . reverse .(take p) . (drop (length prepoint_digits)) . show . round . abs $ (r * 10^p)

useResult :: Either ParseError Expr -> String
useResult (Right expr)  = either id ((showRatio precision) . evaluate) $ replaceVars expr vars
useResult (Left  error) = show error

vars :: Map String Rational
vars = Map.fromList [("pi", 245850922 % 78256779), ("e", 271801 % 99990)]

ioLoop :: IO ()
ioLoop = do done <- isEOF
            if done
              then putStrLn "Quit!"
              else do inp <- getLine
                      let expr = parse exprparser "<stdin>" inp
                      putStrLn . useResult $ expr
                      ioLoop

